﻿// ------------------------------------------------------------------------------------
//
// AnalogInput.CS                                                Ulrich Müller 30.10.09
//                                                               Aspire C# 2005 Express
// Assembly FishFaceTX.DLL : abstracte Klasse AnalogInput
// Basis der Analog-Eingänge
// Klasse A5KInput für Widerstandseingänge, Erben : PhotoResistor, NTC, Potentiometer
// Klasse A10VInput für Spannungseingänge, Erben : ColorSensor
// DistanceSensor : AnalogInput
//
// ------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  /// <summary>
  /// Allgemeiner Analogzugriff am Univiversaleingang
  /// </summary>
  public abstract class AnalogInput : DeviceBase {
    public delegate void Limit(AnalogInput sender, int Value);
    public event Limit ChangedToLow;
    public event Limit ChangedToHigh;
    public event Limit ChangedToNormal;
    public event Limit CurrentValue;

    private int limitLow, limitHigh;
    private int actValue;
    private AnalogState oldState;

    public AnalogInput(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public AnalogInput(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, (int)ctrId, (int)InputName) {
      base.threadInterval = 44;
      limitLow = 0;
      limitHigh = 5001;
    }

    protected override void EventLoop() {
      oldState = AnalogState.Undef;
      while (!tx.NotHalt && !tx.IsESC) {
        actValue = ActualValue;
        if (actValue < limitLow && oldState != AnalogState.Low) {
          oldState = AnalogState.Low;
          if (ChangedToLow != null) ChangedToLow(this, actValue);
        }
        else if (actValue > limitHigh && oldState != AnalogState.High) {
          oldState = AnalogState.High;
          if (ChangedToHigh != null) ChangedToHigh(this, actValue);
        }
        else if (actValue < limitHigh && actValue > limitLow
          && oldState != AnalogState.Normal) {
          oldState = AnalogState.Normal;
          if (ChangedToNormal != null) ChangedToNormal(this, actValue);
        }
        if (CurrentValue != null) CurrentValue(this, actValue);
        Thread.Sleep(base.threadInterval);
      }
    }

    public int LimitHigh {
      get {
        return limitHigh;
      }
      set {
        limitHigh = value;
      }
    }
    public int LimitLow {
      get {
        return limitLow;
      }
      set {
        limitLow = value;
      }
    }

    public abstract int ActualValue {
      get;
    }
    public bool IsLow {
      get {
        return ActualValue < limitLow ? true : false;
      }
    }
    public bool IsHigh {
      get {
        return ActualValue > limitHigh ? true : false;
      }
    }
    public bool IsNormal {
      get {
        int actVal = ActualValue;
        return (actVal >= limitLow && actVal <= limitHigh) ? true : false;
      }
    }
    public AnalogState ActualRange {
      get {
        if (IsLow) return AnalogState.Low;
        if (IsHigh) return AnalogState.High;
        if (IsNormal) return AnalogState.Normal;
        return AnalogState.Undef;
      }
    }

    public void WaitForLow() {
      while (!tx.NotHalt && !tx.IsESC && !IsLow) Thread.Sleep(base.threadInterval);
    }
    public void WaitForHigh() {
      while (!tx.NotHalt && !tx.IsESC && !IsHigh) Thread.Sleep(base.threadInterval);
    }
  }

  /// <summary>
  /// Allgemeine Spannnungsmessung am Universaleingang (Analog 10 Volt)
  /// </summary>
  public class A10VInput : AnalogInput {
    public A10VInput(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public A10VInput(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.deviceType = "A10V";
    }
    public override string ConnectorName {
      get {
        return ((Unv)base.connectorNumber).ToString();
      }
    }
    public override int ActualValue {
      get {
        return tx.GetVoltage((Ctr)base.controllerNumber, (Unv)base.connectorNumber);
      }
    }
  }
  /// <summary>
  /// FarbSensor am Universaleingang (Analog 10 Volt)
  /// </summary>
  public class ColorSensor : A10VInput {
    public ColorSensor(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public ColorSensor(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
    }
  }

  /// <summary>
  /// Allgemeine Analog 5 kOhm Messung am Universaleingang
  /// </summary>
  public class A5KInput : AnalogInput {
    public A5KInput(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public A5KInput(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.deviceType = "A5K";
    }
    public override string ConnectorName {
      get {
        return ((Unv)base.connectorNumber).ToString();
      }
    }
    public override int ActualValue {
      get {
        return tx.GetAnalog((Ctr)base.controllerNumber, (Unv)connectorNumber);
      }
    }
  }
  /// <summary>
  /// Photowiderstand am Universaleingang (Analog 5kOhm)
  /// </summary>
  public class PhotoResistor : A5KInput {
    public PhotoResistor(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public PhotoResistor(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.threadInterval = 44;
    }
  }
  /// <summary>
  /// NTC (Negative Temperature Coefficient) am Universaleingang (Analog 5 kOhm)
  /// </summary>
  public class NTC : A5KInput {
    public NTC(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public NTC(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.threadInterval = 44;
    }
  }
  /// <summary>
  /// Potentiometer am Universaleingang (Analog 5 kOhm)
  /// </summary>
  public class PotentioMeter : A5KInput {
    public PotentioMeter(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public PotentioMeter(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.threadInterval = 44;
    }
  }

  /// <summary>
  /// DistanceSensor am Universaleingang
  /// </summary>
  public class DistanceSensor : AnalogInput {
    /// <summary>
    /// Zuordnen eines Universaleinganges am TX Controller Main
    /// Anlegen eines EventLoops
    /// </summary>
    /// <param name="fc">Zugehörender Instanz für den TX Controller</param>
    /// <param name="WithEvents">Anlegen eines EventLoops</param>
    /// <param name="InputName">Universaleingan</param>
    public DistanceSensor(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    /// <summary>
    /// Zuordnen eines Universaleinganges, Anlegen eines EventLoops
    /// </summary>
    /// <param name="fc">Zugehörender Instanz für den TX Controller</param>
    /// <param name="WithEvents">Anlegen eines EventLoops</param>
    /// <param name="ctrId">Zugehörender TX Controller</param>
    /// <param name="InputName">Universaleingan</param>
    public DistanceSensor(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.deviceType = "Dist";
    }
    /// <summary>
    /// Name des zugehörenden Anschlusses
    /// </summary>
    public override string ConnectorName {
      get {
        return ((Unv)base.connectorNumber).ToString();
      }
    }
    /// <summary>
    /// Aktueller Entfernungswert in cm
    /// </summary>
    public override int ActualValue {
      get {
        return tx.GetDistance((Ctr)base.controllerNumber, (Unv)base.connectorNumber);
      }
    }
  }
}
